@@ -23,53 +23,40 @@
 #include "misc.h"
 #include <QCoreApplication>
 #include <QNetworkRequest>
-#if defined WIN32 && ! defined __MINGW32__
-    #include "win/IeSettings.h"
-    #include "win/Pac.h"
-#endif
-#ifdef __APPLE__
-#include "mac/ProxyDict.h"
-#endif
+#include <QNetworkProxyQuery>
 
-
-static struct NetworkAccessManagerInit
+class NetworkAccessManagerPrivate
 {
-    // We do this upfront because then our Firehose QTcpSocket will have a proxy 
-    // set by default. As well as any plain QNetworkAcessManager stuff, and the
-    // scrobbler
-    // In theory we should do this every request in case the configuration 
-    // changes but that is fairly unlikely use case, init? Maybe we should 
-    // anyway..
+    public:
+    NetworkAccessManagerPrivate();
+    QNetworkProxy userProxy;
+};
 
-    NetworkAccessManagerInit()
+NetworkAccessManagerPrivate::NetworkAccessManagerPrivate() :
+    userProxy( QNetworkProxy( QNetworkProxy::DefaultProxy ) )
+{
+}
+
+
+// TODO: Use a d-pointer on the next SONAME bump
+typedef QHash< const lastfm::NetworkAccessManager *, NetworkAccessManagerPrivate *> NamPrivateHash;
+Q_GLOBAL_STATIC( NamPrivateHash, d_func )
+static NetworkAccessManagerPrivate * d( const lastfm::NetworkAccessManager * nam )
+{
+    NetworkAccessManagerPrivate * ret = d_func()->value( nam, 0 );
+    if ( !ret )
     {
-    #if defined WIN32 && ! defined __MINGW32__
-        IeSettings s;
-        // if it's autodetect, we determine the proxy everytime in proxy()
-        // we don't really want to do a PAC lookup here, as it times out
-        // at two seconds, so that hangs startup
-        if (!s.fAutoDetect && s.lpszProxy)
-        {
-            QUrl url( QString::fromUtf16(s.lpszProxy) );
-            QNetworkProxy proxy( QNetworkProxy::HttpProxy );
-            proxy.setHostName( url.host() );
-            proxy.setPort( url.port() );
-            QNetworkProxy::setApplicationProxy( proxy );
-        }
-    #endif
-    #ifdef __APPLE__
-        ProxyDict dict;
-        if (dict.isProxyEnabled())
-        {
-            QNetworkProxy proxy( QNetworkProxy::HttpProxy );
-            proxy.setHostName( dict.host );
-            proxy.setPort( dict.port );
-
-            QNetworkProxy::setApplicationProxy( proxy );
-        }
-    #endif
+        ret = new NetworkAccessManagerPrivate;
+        d_func()->insert( nam, ret );
     }
-} init;    
+    return ret;
+}
+static void delete_d( const lastfm::NetworkAccessManager * nam )
+{
+    const NetworkAccessManagerPrivate * ret = d_func()->value( nam, 0 );
+    delete ret;
+    d_func()->remove( nam );
+}
 
 
 namespace lastfm 
@@ -81,7 +68,6 @@ namespace lastfm
 lastfm::NetworkAccessManager::NetworkAccessManager( QObject* parent )
                : QNetworkAccessManager( parent )
             #if defined WIN32 && ! defined __MINGW32__
-               , m_pac( 0 )
                , m_monitor( 0 )
             #endif
 {
@@ -98,34 +84,24 @@ lastfm::NetworkAccessManager::NetworkAccessManager( QObject* parent )
 
 lastfm::NetworkAccessManager::~NetworkAccessManager()
 {
-#if defined WIN32 && ! defined __MINGW32__
-    delete m_pac;
-#endif
+    delete_d( this );
 }
 
+void
+lastfm::NetworkAccessManager::setUserProxy( const QNetworkProxy& proxy )
+{
+    d( this )->userProxy = proxy;
+}
 
 QNetworkProxy
 lastfm::NetworkAccessManager::proxy( const QNetworkRequest& request )
 {   
-    Q_UNUSED( request );
+    if ( d( this )->userProxy.type() != QNetworkProxy::DefaultProxy )
+        return d( this )->userProxy;
 
-#if defined WIN32 && ! defined __MINGW32__
-    IeSettings s;
-    if (s.fAutoDetect) 
-    {
-        if (!m_pac) {
-            m_pac = new Pac;
-            if ( !m_monitor )
-            {
-                m_monitor = new InternetConnectionMonitor( this );
-                connect( m_monitor, SIGNAL( connectivityChanged( bool ) ), SLOT( onConnectivityChanged( bool ) ) );
-            }
-        }
-        return m_pac->resolve( request, s.lpszAutoConfigUrl );
-    } 
-#endif
-    
-    return QNetworkProxy::applicationProxy();
+    QList<QNetworkProxy> proxies = QNetworkProxyFactory::systemProxyForQuery( QNetworkProxyQuery( request.url() ) );
+
+    return proxies[0];
 }
 
 
@@ -137,12 +113,8 @@ lastfm::NetworkAccessManager::createRequest( Operation op, const QNetworkRequest
     request.setAttribute( QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache );
     request.setRawHeader( "User-Agent", lastfm::UserAgent );
     
-#ifdef WIN32
     // PAC proxies can vary by domain, so we have to check everytime :(
-    QNetworkProxy proxy = this->proxy( request );
-    if (proxy.type() != QNetworkProxy::NoProxy)
-        QNetworkAccessManager::setProxy( proxy );
-#endif
+    QNetworkAccessManager::setProxy( this->proxy( request ) );
 
     return QNetworkAccessManager::createRequest( op, request, outgoingData );
 }
@@ -152,8 +124,4 @@ void
 lastfm::NetworkAccessManager::onConnectivityChanged( bool up )
 {
     Q_UNUSED( up );
-
-#if defined WIN32 && ! defined __MINGW32__
-    if (up && m_pac) m_pac->resetFailedState();
-#endif
 }
